@@ -876,17 +876,20 @@ class TMod4Impl(TModImpl):
 
         return super().conflicted_modules(module)
 
+    def _emit_restore_instr(self, module):
+        cmds = [f'module restore {module}']
+
+        # Here we append module searchpath removal/addition commands
+        # since 'restore' discards previous module path manipulations
+        for op, mp in self._extra_module_paths:
+            operation = 'use' if op == '+' else 'unuse'
+            cmds += [f'module {operation} {mp}']
+
+        return cmds
+
     def emit_load_instr(self, module):
         if module.collection:
-            cmds = [f'module restore {module}']
-
-            # Here we append module searchpath removal/addition commands
-            # since 'restore' discards previous module path manipulations
-            for op, mp in self._extra_module_paths:
-                operation = 'use' if op == '+' else 'unuse'
-                cmds += [f'module {operation} {mp}']
-
-            return cmds
+            return self._emit_restore_instr(module)
 
         return super().emit_load_instr(module)
 
@@ -1004,6 +1007,17 @@ class LModImpl(TMod4Impl):
         # we forcefully unload everything.
         self.execute('--force', 'purge')
 
+    def emit_load_instr(self, module):
+        if module.collection:
+            return self._emit_restore_instr(module)
+
+        cmds = []
+        if module.path:
+            cmds.append(f'module use {module.path}')
+
+        cmds.append(f'module load {module.fullname}')
+        return cmds
+
 
 class NoModImpl(ModulesSystemImpl):
     '''A convenience class that implements a no-op a modules system.'''
