@@ -12,6 +12,7 @@ import functools
 import itertools
 
 import reframe.core.namespaces as namespaces
+import reframe.utility as utils
 from reframe.core.exceptions import ReframeSyntaxError
 
 
@@ -45,6 +46,18 @@ class TestParam:
                 return x
 
         self.values = tuple(values)
+
+        # Validate the filter_param argument
+        try:
+            valid = utils.is_trivially_callable(filter_params, non_def_args=1)
+        except TypeError:
+            raise TypeError(
+                'the provided parameter filter is not a callable'
+                ) from None
+        else:
+            if not valid:
+                raise TypeError('filter function must take a single argument')
+
         self.filter_params = filter_params
 
 
@@ -124,9 +137,18 @@ class ParamSpace(namespaces.Namespace):
 
         local_param_space = getattr(cls, self.local_namespace_name)
         for name, p in local_param_space.items():
-            self.params[name] = (
-                p.filter_params(self.params.get(name, ())) + p.values
-            )
+            try:
+                filt_vals = p.filter_params(self.params.get(name, ()))
+            except Exception:
+                raise
+            else:
+                try:
+                    self.params[name] = (tuple(filt_vals) + p.values)
+                except TypeError:
+                    raise ReframeSyntaxError(
+                        f"'filter_param' must return an iterable "
+                        f"(parameter {name!r})"
+                    ) from None
 
         # If any previously declared parameter was defined in the class body
         # by directly assigning it a value, raise an error. Parameters must be
