@@ -14,7 +14,8 @@ import sys
 
 import reframe.utility as util
 import reframe.core.warnings as warn
-from reframe.core.deferrable import deferrable, _DeferredExpression
+from reframe.core.deferrable import (deferrable, _DeferredExpression,
+                                     _DeferredPerformanceExpression)
 from reframe.core.exceptions import SanityError
 
 
@@ -40,6 +41,29 @@ def _open(filename, *args, **kwargs):
         raise SanityError(f'{filename}: {e.strerror}')
 
 
+def make_performance_function(func, unit, *args, **kwargs):
+    '''Convert a callable or deferred expression into a performance function.
+
+    If ``func`` is a deferred expression, the performance function will be
+    built by extending this deferred expression into a deferred performance
+    expression. Otherwise, a new deferred performance expression will be
+    created from the function :func:`func`. The argument ``unit`` is the unit
+    associated with the deferrable performance expression, and ``*args`` and
+    ``**kwargs`` are the arguments to be captured by this deferred expression.
+    See
+    :doc:`deferrable functions reference </deferrable_functions_reference>`
+    for further information on deferrable functions.
+
+    .. versionadded:: 3.8.0
+    '''
+    if isinstance(func, _DeferredExpression):
+        return _DeferredPerformanceExpression.construct_from_deferred_expr(
+            func, unit
+        )
+    else:
+        return _DeferredPerformanceExpression(func, unit, *args, **kwargs)
+
+
 # Create an alias decorator
 def sanity_function(func):
     warn.user_deprecation_warning(
@@ -886,15 +910,32 @@ def defer(x):
     return x
 
 
-def evaluate(expr):
+def evaluate(expr, cache=False):
     '''Evaluate a deferred expression.
 
     If ``expr`` is not a deferred expression, it will be returned as is.
+    If ``expr`` is a deferred expression and ``cache`` is ``True``, the
+    results of the deferred expression will be cached and subsequent calls
+    to :func:`evaluate` on this deferred expression (when ``cache=False``)
+    will simply return the previously cached result.
+
+    :param expr: The expression to be evaluated.
+    :param cache: Cache the result of this evaluation.
+
+    .. note::
+       When the ``cache`` argument is passed as ``True``, a deferred
+       expression will always be evaluated and its results will be re-cached.
+       This may replace any other results that may have been cached in
+       previous evaluations.
 
     .. versionadded:: 2.21
+
+    .. versionchanged:: 3.8.0
+       The ``cache`` argument is added.
     '''
+
     if isinstance(expr, _DeferredExpression):
-        return expr.evaluate()
+        return expr.evaluate(cache=cache)
     else:
         return expr
 
